function [x,y,s,sw,p,t,h]=lp_rmpc(A, b, c, swin, pin)

% version: rmpc v3.0
% USAGE:
% [x,y,s,sw,p,t,h]=lp_rmpc(A, b, c, swin, pin)
%
% A constraint reduced variant of the Mehrotra Predictor-Corrector
% primal-dual interior point algorithm for linear programming
%
% Luke Winternitz
% Initial version 9/2007
% Revised most recently 3/2009
% lukewinternitz at gmail dot com
%
% inputs:
% A,b,c: problem data for LP in standard form
% swin:  structure of algorithm switches, see rmpc_get_defaults.m for description of fields
%        unfilled entries will get default values asigned
% pin:   structure of algorithm parameters, see get rmpc_get_defaults.m for description of fields
%        unfilled entries will get default values asigned
% outputs:
% x,y,s: computed primal-dual solution upon sucessful termination
% sw   : structure of algorithm switches used in algorithm
% p    : structure of algorithm parameters used in algorithm
% t    : output structure: example
%            iter_avg: average iteration time
%            factor_avg: averate time to factor normal matrix
%            total: total cpu time to solve
%            niter: number of iterations
%            status: status bit 1=success -1=fail
%            sblockdata:
%            Mtotal: total number of constraints used in all iterations (divide by t.niter to get average size of Q)
% h    : output iteration history structure
%            dual: dual objective values, array of size t.niter by 1
%
% Reference:
% L.B. Winternitz, S.O. Nicholls, A.L. Tits and D.P. O'Leary,
% A constraint reduced variant of Mehrotra's Predictor-Corrector Algorithm, 2007.
% Unpublished technical report, available on Optimization Online
%
%
% depends on:
% rmpc_get_defaults.m:  sets default parameter/switch values
% pd_project.m          attempts projection of advanced iterate onto optimal face 
%                       (or basis inversion if nondegenerate solution is approached)
% getinitialQ.m.m       constraint selection function
% decluster.m:          declusters constraint set Q: used in getinitialQ.m
%                       sometimes useful for chebyshev problems (current implementation is pretty inefficient)
%                        (not used anymore in v3)

%get problem size
[m,n]=size(A);

%get parameter and switch defaults
[p,sw]=rmpc_get_defaults(m,n);
%update with input values

if(~isempty(pin))
    flds=fieldnames(pin);
    for i=1:length(flds)
        p.(flds{i})=pin.(flds{i});
    end
end

if(~isempty(swin))
    flds=fieldnames(swin);
    for i=1:length(flds)
        sw.(flds{i})=swin.(flds{i});
    end
end

%for convienience add these fields
p.m=m;
p.n=n;

%optimal dual value if known
dstar=p.dstar;


%display switches and parameters
swin
pin

%ignore near singularity complaints, etc.
warning off;


%gridded constraints
if(p.Mgrid>0)
    igrid=floor(1:(n/p.Mgrid):n)';
else
    igrid=[];
end

%box constraints
if(sw.box)
    ibox=(1:2*m)';
else
    ibox=[];
end
%constraints always in Q
Qfixed=union(igrid,ibox);

%number of additional constraints to add beyond box constraints and basic
%most active constraints
p.M1=p.M-p.M0-sw.box*2*m-p.Mgrid;


%select an initial point
%this allows various options
%[x,y,s,p]=lp_rmpc_get_initpt(sw.initial_pdpoint, A, b, c, p);
%but mainly just use user provided point these days
y=pin.y0;
s=pin.s0;
x=p.xi*ones(n,1);



% allocate storage for iteration histories
% muhist=zeros(p.maxiter,1);
% primal=zeros(p.maxiter,1);
dual=zeros(p.maxiter,1);

%display header
if(sw.disp)
    fprintf('-------------------------------------Luke Winternitz rMPC------------------------------------------------------------------------------------------------------------------------------\n');
    fprintf('\nIt#    mQ titer tform   Nxlb lb^0.5     lb^3     min(x+) min(xisi/mu) limit min? proj  itp igmin safeg gamma  tstep Mstep termcrit ');
    fprintf('     gap    d-resid   p-resid  pQ-resid  deltacent sigmanew  primal-obj dual-obj rel(d-dstar)rankM/m condM\n');
    fprintf('---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------\n');
end

%anything smaller than tolneg is considered negative
%tolneg=0;
tolneg=-1e-14;

%initialize a bunch of variables
gamma=0;    %mixing parameter

%compute these once
normb=norm(b);
normc=norm(c);

%setup output/timer structure
t.iter_avg = 0;
t.factor_avg = 0;
t.total  = 0;
t.niter = 0;
t.status = 0; %assume failure
t.sblockdata = zeros(p.maxiter, 4);
t.Mavg=0;
t.Mtotal=0;
t.Mmax=0;
t.it_10m2=-1;

%allocate storage for various vectors
xtilde_aff=zeros(n,1);
xtilde_aff_minus=zeros(n,1);
xtilde_mpc=zeros(n,1);
xtilde_mpc_minus=zeros(n,1);
xplus=zeros(n,1);
ccrhs=zeros(n,1);
dxaff=zeros(n,1);
dx=zeros(n,1);
dxcc=zeros(n,1);
dsaff=zeros(n,1);
dscc=zeros(n,1);

sigma=0;
ncx=0;
ncy=0;

%random stuff
R=eye(m);

%scaling factors for |dyaff|
nay0=m;
nay0_2=1;

nn=(1:n)';
Q=Qfixed;



%constraint valuation array
v=zeros(n,1);

%main iteration loop
for k=1:p.maxiter

    %start iteration timer
    t_iter=cputime;

    %compute the infeasibilty gaps for the primal-dual equality constraints
    cmAty=c-(A'*y);
    rc=(s-cmAty);
    rb=(A*x-b);

    %primal and dual objective values
    pval=c'*x;
    dval=b'*y;
    dmdstar=abs(dstar-dval)/(abs(dstar)+1);

    %store dual objective values
    dual(k)=dval;

    %norms of residuals
    nrb=norm(rb);
    nrc=norm(rc);

    %compute the duality gap of the current iterate
    gap=abs(pval-dval);

    %duality measure
    mu=(x'*s)/n;
    deltacent=norm(x.*s/mu - ones(n,1));

    %relative gaps/infeasibilites this makes the termination
    %less scaling dependent. These are as suggested in SDPT3.
    relgap = gap/(1+abs(pval)+abs(dval));
    pinfeas = nrb/(1+normb);
    dinfeas = nrc/(1+normc);

    %compute stopping criteria
    %termcrit = max([nrb/(1+norm(x)),nrc/(1+norm(s)), gap/(1+abs(pval))]);
    nx=norm(x);
    ns=norm(s);
    termcrit = max([nrb/(1+nx), norm(min(x,0))/(1+nx), nrc/(1+ns), norm(min(s,0))/(1+ns), gap/(1+abs(dval))]);


    if(termcrit<p.tol_conv)
        t.status=1; %record success;
        break;
    end

    %log number of iterations to get to 10^-2
    if(termcrit<1e-2 & t.it_10m2==-1)
        t.it_10m2=k-1; 
    end



    %%choose initial subset of constraints to process
    [Q,Qc,mQ,Qmins,v,ssort,isort]=getinitialQ(cmAty, x, termcrit, sw, p, Qfixed, v);

    

    %compute partial primal residual-does this goes to zero faster than the full residual?
    nrbQ=norm(b-A(:,Q)*x(Q));

    %solve linear system for the search direction
    %form the [m x m] normal matrix, M=A*D^2*A'
    t_factor = cputime;
    switch sw.chol


          
        case 'chol_iter'

            while(1)

                t_form=cputime;
                D2=spdiags(x(Q)./s(Q), 0, mQ, mQ);
                M=A(:,Q)*D2*A(:,Q)';
                t_form=cputime-t_form;

                [R,degensubmat]=chol(M);
                

                
                if(degensubmat==0)
                    break;
                else
                    if(mQ==n)
                        %fall back on cholinf?
                        R=cholinc(sparse(M), 'inf');
                        break;
                        %error('fact:chol:nsing', 'cholesky factorization
                        %failed with all constraints!');
                    end

                    Q=union(Q,isort(1:min(n,2*mQ)));
                    Qc=setdiff(nn, Q);
                    mQ=length(Q);

                    if(sw.disp)
                        fprintf('Doubled constraint set.\n');
                    end
                end
            end

            if(sw.computeMrank)
                rankM=rank(full(M));
                condM=cond(full(M));
            else
                rankM=0;
                condM=0;
            end

            %solve for the affine scaling direction
            dyaff = R\(R'\b);
            dsaff = -(A'*dyaff); %strangely this is MUCH-MUCH faster than -A'*dyaff;
            dxaff(Q) = -x(Q) - D2*dsaff(Q);

        otherwise
            error('invalid sw.chol');
    end %sw.chol
    t_factor=cputime-t_factor;
    t.factor_avg = ( (k-1)*t.factor_avg + t_factor )/k;

  

    %find boundary of feasible region along affine scaling direction,
    %use a separate step-size for the primal (x) and dual (s) variables
    isneg = find(dsaff<tolneg);
    ixneg = Q(dxaff(Q)<tolneg);

    %primal step-to-boundary
    if(~isempty(ixneg))
        pstp2bdy = -x(ixneg)./dxaff(ixneg);
    else
        pstp2bdy=inf;
    end

    %dual step-to-boundary
    if(~isempty(isneg))
        dstp2bdy = -s(isneg)./dsaff(isneg);
    else
        dstp2bdy=inf;
    end

    %common step size or not
    if(sw.common_pdstep)
        tpaff=min(min([pstp2bdy; dstp2bdy]),1);
        tdaff=tpaff;
    else
        tpaff = min(min(pstp2bdy),1);
        tdaff = min(min(dstp2bdy),1);
    end

    xtilde_aff(Q)=x(Q)+dxaff(Q);
    xtilde_aff_minus(Q)=min([zeros(mQ,1), xtilde_aff(Q)], [], 2); %componentwise negative part of xtilde_aff
    

    %skip computation of the cent-corr direction (to be fair) if it is a pdas algorithm
    if ~strcmp(sw.alg(1:4),'pdas')

        %determine sigma depending on how effective the affine scaling
        %direction was in reducing the duality parameter mu, or in how long the
        %max feasible step was along the affine scaling direction
        %modification of the standard heuristic (mu_aff/mu)^3
        sigma=(1-min(tpaff, tdaff))^3;


        %solve linear system for the combined centering/correction step
        %using normal equations form
        switch sw.chol

            case 'chol_iter'

                %form the right hand side for the combined centering/corrector step
                muQ=x(Q)'*s(Q)/mQ;
                ccrhs(Q) = ( sigma*muQ*ones(mQ,1)  - dxaff(Q).*dsaff(Q) )./s(Q);
                fcc = -A(:,Q)*ccrhs(Q);

                %solve for the combined centering/corrector direction
                dycc = R\(R'\fcc);
                dscc = -(A'*dycc); %this is ridiculously faster than -A'*dycc;
                dxcc(Q) = ccrhs(Q) - D2*dscc(Q);

        end %sw.chol
        
        %compute some things
        ncy=norm(dycc);
        ncx=norm(dxcc);


    end%skip cc computation;

    %compute some things
    nay=norm(dyaff);
    naxt=norm(xtilde_aff(Q));
   
    


    %form the total search direction made of three parts: affine scaling,
    %centering, and corrector directions. Is there a reason not to make a
    %more free linear combination of these terms?

    %tells which safeguard set gamma see below
    igmin=-1;
    %this switch controls how gamma is formed; mpc++ is variant with convergence guarantee.
    switch(sw.alg)
        case {'pdas_TAW', 'pdas_phi0'}
            dx(Q)=dxaff(Q);
            dy=dyaff;
            ds=dsaff;
        case 'mpc'
            %mpc direction
            dx(Q)=dxaff(Q)+dxcc(Q);
            dy=dyaff+dycc;
            ds=dsaff+dscc;
        case 'mpc+'
            %damped mpc direction to enforce ascent only
            vaff = b'*dyaff;
            vcc  = b'*dycc;

            if(vcc<tolneg)
                gamma = min(1, (1-p.theta_ascent)*abs(vaff)/abs(vcc));
            else
                gamma = 1;
            end
            %mpc direction
            dx(Q)=dxaff(Q)+gamma*dxcc(Q);
            dy=dyaff+gamma*dycc;
            ds=dsaff+gamma*dscc;
        case 'mpc++'
            %damped mpc direction to enforce ascent and enforce
            %dy^a -> 0 implies gamma*dy^c -> 0 and gamma sigma mu ->0
            %ascent part
            vaff = b'*dyaff;
            vcc  = b'*dycc;

            if(vcc<tolneg)
                gammabar = min(1, (1-p.theta_ascent)*abs(vaff)/abs(vcc));
            else
                gammabar = 1;
            end

            [gamma, igmin] = min([gammabar, p.R1*nay/ncy, p.R2*nay/(sigma*mu), p.R3*naxt/ncx]);
            %[gamma, igmin] = min([gammabar, p.R1*nay/ncy, p.R2*nay/(sigma*mu)]);

            dx(Q)=dxaff(Q)+gamma*dxcc(Q);
            dy=dyaff+gamma*dycc;
            ds=dsaff+gamma*dscc;

        otherwise
            error('invalid sw.alg');
    end  %switch(sw.alg)

    %compute the full step
    xtilde_mpc(Q)=x(Q)+dx(Q);
    %componentwise negative part of xtilde_mpc (not needed)
    xtilde_mpc_minus(Q)=min([zeros(mQ,1), xtilde_mpc(Q)], [], 2);

    %do line search to find boundary of feasible region along total search direction
    isneg=find(ds<tolneg);
    if(~isempty(isneg))
        dstp2bdy = -s(isneg)./ds(isneg);
    else
        dstp2bdy = inf;
    end

    [tdmpc,idlimit]=min([1;dstp2bdy]);


    %gather some info on blocking constraint
    if(idlimit==1)
        steplimitQ=-1; %step of 1 taken, not limited
        isblock_min=-1;
    else %if limited get some data on the blocking constraint

        isblock=isneg(idlimit-1); %index of blocking constraint

        %check if it is a local min of the slack function
        if(isblock==1)
            isblock_min=s(isblock)<s(isblock+1);
        else
            if(isblock==n)
                isblock_min=s(isblock)<s(isblock-1);
            else
                isblock_min=s(isblock)<min(s(isblock+1), s(isblock-1));
            end
        end
        
        if(ismember(isblock,Q))
            steplimitQ=0;
        else
            steplimitQ=1;
        end
    end



    %put in output structure
    t.sblockdata(k,:)=[termcrit, tdmpc, steplimitQ, isblock_min];


    %next stuff is for enforcing the condition t_d^mpc >= \zeta t_d^aff
    %needed for convergence proof

    %indicate whether we need to limit gamma to enforce the condition
    flag_safeguard=(tdmpc<p.theta_stepsize*tdaff);

    %switch on how it will be enforced
    switch sw.safeguard
        %fall back on affine-scaling direction where condition above certainly holds
        case 'discont'
            if(flag_safeguard)
                gamma=2;
                dx(Q)=dxaff(Q);
                dy=dyaff;
                ds=dsaff;
                tpmpc=tpaff;
                tdmpc=tdaff;
            end
        case 'cont' %method described in rMPC paper, somewhat more "continuous"
            if(flag_safeguard)
                %redefine the gamma so that steplength condition is satisfied
                gamma=(1-p.theta_stepsize)*tdmpc/((1-p.theta_stepsize)*tdmpc+(p.theta_stepsize*tdaff-tdmpc))*gamma;
                tdmpc=p.theta_stepsize*tdaff;
                %reform search directions
                dx(Q)=dxaff(Q)+gamma*dxcc(Q);
                dy=dyaff+gamma*dycc;
                ds=dsaff+gamma*dscc;
                if(sw.safeguard_redols)
                    %redo line search to find boundary of feasible region along total search direction
                    isneg=find(ds<tolneg);
                    if(~isempty(isneg))
                        dstp2bdy = -s(isneg)./ds(isneg);
                    else
                        dstp2bdy = inf;
                    end
                    tdmpc=min([1;dstp2bdy]);
                end
            end
        otherwise
            error('invalid sw.safeguard');
    end %sw.safeguard


    %find primal step to boundary
    ixneg=Q(dx(Q)<tolneg);
    if(~isempty(ixneg))
        pstp2bdy = -x(ixneg)./dx(ixneg);
    else
        pstp2bdy = inf;
    end
    tpmpc = min([1;pstp2bdy]);

    tp=tpmpc;
    td=tdmpc;

    %stepsize rules that allow quadratic convergence to take place
    [tp,itp]=max([p.beta*tp, tp-(nay/nay0_2)]);
    [td,itd]=max([p.beta*td, td-(nay/nay0_2)]);

   

    %update dual variables
    y=y+td*dy;
    sminus=s;%store last s
    s=s+td*ds;
    %s=c-A'*y;  %above is cheaper and mathematically equivalent,
    %but this may be better numerically?

    %keep s off boundary a tiny bit for numerical problem avoidance
    if(sw.soft_constr)
        s=max(p.tol_soft_constr, s);
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % update primal variables
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    


    

    switch sw.alg
        case 'pdas_TAW'
            xplusaff = -spdiags(x./sminus, 0, n, n)*dsaff;
            xlb0=inf;
            xlb1 = p.lbmult*( (nay/nay0)^p.lbexp + norm(min(xplusaff,0))^p.lbexp );
            lbaff=min(xlb1,p.xlb)*ones(n,1);
            [x, ixupdate]=max([lbaff, xplusaff],[],2);
            
        case 'pdas_phi0'
             %compute the damped primal update along search direction
            xplus(Q) = xtilde_aff(Q);

            %Compute lower bound appearing in x update
            %trying various things here
            xlb1 = p.lbmult*( (nay/nay0)^p.lbexp + norm(xtilde_aff_minus(Q))^p.lbexp );
            xlb2 = p.xlb*ones(mQ,1);
            [lb, whichlb]=min([xlb1*ones(mQ,1), xlb2],[],2);

            %update Q variables
            [x(Q), ixupdate]=max([lb, xplus(Q)], [], 2);

            %update Qc variables to center them
            muplus=(s(Q)'*x(Q))/mQ;

            %new clipped update of x(Qc)
            x(Qc)=min(muplus./s(Qc), p.R4*naxt);

            
        otherwise
            %compute the damped primal update along search direction
            xplus(Q) = x(Q) + tp*dx(Q);

            %Compute lower bound appearing in x update
            %trying various things here
            %xlb0 = p.lbmult*( norm(dyaff)^0.5+norm(xtilde_aff_minus)^0.5 );
            xlb0 = inf;
            xlb1 = p.lbmult*( (nay/nay0)^p.lbexp + norm(xtilde_aff_minus(Q))^p.lbexp );
            %xlb1 = p.lbmult*( (nay)^p.lbexp + norm(xtilde_aff_minus(Q))^p.lbexp );
            xlb2 = p.xlb*ones(mQ,1);
            [lb, whichlb]=min([xlb0*ones(mQ,1), xlb1*ones(mQ,1), xlb2],[],2);

            %update Q variables
            [x(Q), ixupdate]=max([lb, xplus(Q)], [], 2);

            %update Qc variables to center them
            muplus=(s(Q)'*x(Q))/mQ;

            %new clipped update of x(Qc)
            x(Qc)=min(muplus./s(Qc), p.R4*naxt);
    end

    %count the number of x components that are limited by the lower bound
    %rather than taking the normal mpc update
    Nxlb=sum(ixupdate==1);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %update timers
    t_iter=cputime-t_iter;
    t.iter_avg = ( (k-1)*t.iter_avg + t_iter )/k;
    t.total=t.total+t_iter;


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Mehrotra-Ye finite termination procedure
    % attempt to project the current iterate onto the optimal face
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    proj_opt=-1;
    %the second condition is a simplistic trigger mechanism, there are
    %better ways for sure, some references are pointed to in Kaliski-Ye.
    if(sw.attempt_proj && termcrit<p.proj_thresh)

        %estimated B/N partition
        Bp=find(x>=s);
        Np=find(s>x);
        mp=length(Bp);

        if(mp<=m)
            %if(termcrit<1e-2 && mp<=m)
            [xp, yp, sp, proj_opt]=pd_project(x,y,s, Bp,Np,A,b,c);
            %report success after printf
        else
            proj_opt=-2; %signifies not attempted
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    %set up iteration display
    display_data=full([k, mQ, t_iter, t_form, Nxlb, p.xlb/p.lbmult, xlb1/p.lbmult, min(xplus(Q)), n*min(x.*s)/(x'*s), steplimitQ, isblock_min, proj_opt, itp+itd, igmin, flag_safeguard, gamma, tdaff, tdmpc, termcrit, ...
        gap, nrc, nrb, nrbQ, deltacent, sigma, pval, dval, dmdstar, rankM, m, condM]);
    if(any(isnan(display_data)))
        t.status=-1;
        break;
    end
    if(isinf(xlb1))
        t.tstatus=-1;
        break;
    end

    %count total number of contraints used, for computing average constraint size
    t.Mtotal=t.Mtotal+mQ;
    t.Mmax=max(t.Mmax, mQ);


    %display iteration info
    if(sw.disp)
        fprintf('%3d %5d %5.3f %5.3f %5d %5.3e %5.3e %5.3e %5.3e %+5d %+5d %+5d %5d %5d %5d %5.3f %5.3f %5.3f %5.3e %5.3e %5.3e %5.3e %5.3e %5.3e %5.3e %+5.3e %+5.3e %5.3e %d/%d %5.2e\n', display_data);
    end

    %plot slack function at each iteration (interesting to watch sometimes)
    if(sw.plotslacks)
        if(sw.box)
            %dont plot the slack in box constraints if box was added
            %note Q is associated to sminus since s has already been
            %updated and Q has not. Just wanted to collect display functions down here.
            plot(1:(p.n-2*p.m), sminus(2*p.m+1:p.n), 'b');
        else
            plot(sminus, 'b');
        end
        hold on;
        autoaxis=axis;
        axis([1,p.n-sw.box*2*p.m,autoaxis(3:4)]);
        plot(Q-sw.box*2*p.m, sminus(Q), 'ro', 'LineWidth', 2, 'MarkerSize', 6);
        plot(Qmins-sw.box*2*p.m, sminus(Qmins), 'go', 'LineWidth', 2, 'MarkerSize', 6);
        hold off;
        drawnow;
        pause(0.01);
    end


    %if finite termination succeeded, put down here because still wanted to see
    %other display stuff before terminating.
    if(proj_opt==1)
        x=xp;s=sp;y=yp;
        if(sw.disp)
            fprintf('Projection Succeeded! Terminating\n');
            t.status=1;
        end
        break;
    end

end %main iteration loop

%record number of iterations
if(k==p.maxiter)
    t.niter=p.maxiter;
else    
    t.niter=k-1; %since termination criterion is checked at beginning of iteration (k has been incremented)
end
t.Mavg=t.Mtotal/t.niter;

%truncate some arrays
dual=dual(1:k-1);
t.sblockdata=t.sblockdata(1:k-1,:);

%print some stuff
if(sw.disp)
    fprintf('final state\n')
    switch(t.status)
        case -1
            fprintf('FAILURE: NANINF\n');
            t.iter=inf;
            t.iter_avg=inf;
            t.factor_avg = inf;
            t.total  = inf;
            t.niter = inf;
        case  0
            fprintf('FAILURE: MAXIT\n');
            t.iter=inf;
            t.iter_avg=inf;
            t.factor_avg = inf;
            t.total  = inf;
            t.niter = inf;
        case 1
            fprintf('SUCCESS!\n');
        otherwise
            fprintf('INVALID STATUS CODE\n');

    end
    fprintf('-----------------------------------------------------------------\n')
    fprintf('number of iterations: %3d\n', full(k));
    fprintf('primal objective value: %e\n', full(pval));
    fprintf('dual objective value: %e\n', full(dval));
    fprintf('duality gap: %e\n', full(gap));
    fprintf('relative duality gap: %e\n', full(gap/(1+abs(pval))));
    fprintf('error-measure: %e\n', full(termcrit));
    fprintf('relative primal infeasibility: %e\n', full(pinfeas));
    fprintf('primal-Q infeasibility: %e\n', full(norm(A(:,Q)*x(Q)-b)));
    fprintf('relative dual infeasibility: %e\n', full(dinfeas));
    fprintf('total time: %f\n', full(t.total));
    fprintf('time per iteration: %f\n', full(t.iter_avg));
end

% muhist=muhist(1:k);
% primal=primal(1:k);
% dual=dual(1:k);

%iteration history output structure
%h=struct('muhist', muhist, 'pobj', primal, 'dobj', dual);
h.dual=dual;

%turn warnings back on
warning on;
